﻿using CoreLibrary.Exceptions;
using CoreLibrary.Persistance;
using CoreLibrary.Evenements;
using CoreLibrary.Regles;
using CoreLibrary.Statistiques;
using System.Runtime.Serialization;
using CoreLibrary.Core;

namespace CoreLibrary.Joueurs
{
    [DataContract]
    [KnownType(typeof(ReglesClassiques))]
    [KnownType(typeof(ReglesDifficiles))]
    public class Joueur : IEstPersistant
    {
        public event EventHandler<JoueurSeConnecterEventArgs>? JoueurSeConnecter;

        private void QuandJoueurSeConnecter(Joueur joueur) => JoueurSeConnecter?.Invoke(this, new JoueurSeConnecterEventArgs(joueur));
        
        [DataMember]
        private Dictionary<(IRegles, Statistique), int> statistiques = new Dictionary<(IRegles, Statistique), int>();

        [DataMember]
        public string Nom { get; private set; } = "";
        public bool EstConnecte { get; private set; } = false;

        public Joueur()
        {
        }

        public Joueur(string nom)
        {
            Nom = nom;
        }

        public void SeConnecter(Joueur joueur)
        {
            if (EstConnecte)
                throw new JoueurDejaConnecteException(this);

            QuandJoueurSeConnecter(joueur);
        }

        public void JouerPartie(Partie partie)
        {
            partie.PartieDemanderJoueurJouer += QuandDemanderJoueurJouer;
        }

        public virtual void QuandDemanderJoueurJouer(object? sender, PartieDemanderJoueurJouerEventArgs e)
        {
            if (e.Nom != Nom)
                return;
        }

        public override string ToString() => Nom;

        public int Statistique(IRegles regles, Statistique statistique) => statistiques.GetValueOrDefault((regles, statistique),0);

        public void IncrementerStatistique(IRegles regles, Statistique statistique) => 
            statistiques[(regles, statistique)] = Statistique(regles, statistique) + 1;
    }
}
